//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  RPTPROC.C - MicroLan repeater packet processing.
//
//  Version: 1.00
//

// define so only declared once
#define MLANPKT

#include <stdio.h>
#include <string.h>
#include "mlanpkt.h"

// extern Link, Transport and Network MicroLAN functions
extern uchar MLanTouchReset(void);
extern uchar MLanDelay(uchar);
extern uchar MLanSearch(void);
extern uchar MLanAccess(void);
extern uchar MLanOverdriveAccess(void);
extern uchar MLanBitBlock(uchar, uchar *);
extern uchar MLanByteBlock(uchar, uchar *);
extern void  UpdateMLanMode(uchar);

// other external functions
extern void  MLanOutboundSendEvent(uchar *);

// local functions
int   MLanProcessPacket(void);
uchar MLanProcessCommand(uchar, uchar, uchar *);
uchar DataCommand(uchar, uchar, uchar, uchar *, uchar *);
uchar SingleResult(uchar, uchar); 
int   CheckOutboundSpace(uchar);
uchar MLanRepeaterReset(void);
int   ScanGETBUF(uchar *, uchar);

//----------------------------------------------------------------------
// Process the inbound frame buffer 'inbound' and creates an outbound 
// reponse frame 'outbound'.  This function returns TRUE if the frame 
// was processed without error and FALSE otherwise.
//
// Returns: TRUE:  commands processes without error
//          FALSE: an error occured during the command processing
//
int MLanProcessPacket(void)
{
   static uchar DidRepeaterReset = FALSE;
   uchar data_length;
   uchar inbound_length, in_cnt = 0, do_command = 0;
   uchar *data_bytes;

   // retrieve the length of the inbound frame from 
   // the first byte
   inbound_length = inbound[in_cnt++];

   // loop to process through the inbound packet
   while (in_cnt < (inbound_length + 1))
   {
      // retrieve an inbound buffer byte, command 
      last_cmd = inbound[in_cnt++];

      do_command = TRUE;
      
      // check for multi-byte command
      if (!(SINGLE_BYTE_CMD & last_cmd))
      {
         // retrieve an inbound buffer byte, data length 
         data_length = inbound[in_cnt++];

         // verify that data_length bytes available in inbound buffer
         if ((in_cnt + data_length) > (inbound_length + 1)) 
         {
            // send error to the outbound buffer
            SingleResult(CMD_ERROR, RET_END_OF_INBOUND);
            // remember last cmd and result
            last_cmd = CMD_ERROR;
            last_cmd_return = RET_END_OF_INBOUND;
            do_command = FALSE; 
         }

         // get a pointer to the data bytes
         data_bytes = &inbound[in_cnt];

         // adjust the inbound buffer count
         in_cnt += data_length;
      }
      // single-byte command so check for space for 2 bytes
      else if (last_cmd != CMD_GETBUF)
         do_command = CheckOutboundSpace(2);

      // process the individual command
      if (do_command)
         last_cmd_return = MLanProcessCommand(last_cmd, data_length, data_bytes);
      
      // check return_code for halt condition
      if ((last_cmd_return != RET_SUCCESS) &&
          (last_cmd_return != RET_END_SEARCH))
      {
         // check for CMD_GETBUF command in remainder of inbound
         if (ScanGETBUF(&inbound[in_cnt], (uchar)(inbound_length - in_cnt + 1)))
         {
            last_cmd = CMD_GETBUF;
            last_cmd_return = RET_SUCCESS;
            MLanOutboundSendEvent(outbound);
         }
         return FALSE;
      }
   }
 
   return TRUE;
}


//----------------------------------------------------------------------
// Process the command and place the result in the outbound reponse
// frame.  This function returns the return code result of command.
//
//  'cmd'         - the command code to be processed
//  'data_length' - number of data bytes if mult-byte command
//  '*data_bytes' - pointer to the begining of data bytes if multi-byte
//                  command, (points into inbound buffer)
//
// Returns: return code result of command
//
uchar MLanProcessCommand(uchar cmd, uchar data_length, uchar *data_bytes)
{
   uchar return_code;

   // switch on the command type
   switch (cmd)
   {
      // single-byte commands
      // MicroLan Reset
      case CMD_ML_RESET:
         // call the MicroLan reset function 
         return_code = SingleResult(cmd, MLanTouchReset());
         break;
      // peform a search on the MicroLan
      case CMD_ML_SEARCH:
         return_code = SingleResult(cmd, MLanSearch());
         break;
      // peform an 'access', match command on ID in DATA_ID
      case CMD_ML_ACCESS:
         return_code = SingleResult(cmd, MLanAccess());
         break;
      // peform an 'overdrive access', match command on ID in DATA_ID 
      // and switch to overdrive communication speed 
      case CMD_ML_OVERDRIVE_ACCESS:
         return_code = SingleResult(cmd, MLanOverdriveAccess());
         break;
      // reset the repeater
      case CMD_RESET:
         return_code = SingleResult(cmd, MLanRepeaterReset()); 
         break;
      // send the outbound buffer to the host
      case CMD_GETBUF:
         MLanOutboundSendEvent(outbound);
         return_code = RET_SUCCESS;
         break;
      // delay (nothing added to outbound)
      case CMD_DELAY:
         // make sure 1 data byte provide as the delay length
         if (data_length == 1)
            return_code = MLanDelay(data_bytes[0]);
         else
            return_code = SingleResult(CMD_ERROR, RET_WRITE_ONLY);
         break;
      // multi-byte commands (DATA)
      // read/write DATA_ID register
      case DATA_ID:
         return_code = DataCommand(cmd,data_length,SIZE_DATA_ID,data_bytes,&id[0]);
         break;
      // read/write DATA_SEARCH_STATE register
      case DATA_SEARCH_STATE:
         return_code = DataCommand(cmd,data_length,SIZE_DATA_SEARCH_STATE, 
                                  data_bytes,&search_state[0]);
         // if writing search state clear the last device flag
         if (data_length)
            LastDeviceFlag = 0;
         break;
      // read/write DATA_SEARCH_CMD register
      case DATA_SEARCH_CMD:
         return_code = DataCommand(cmd,data_length,SIZE_DATA_SEARCH_CMD, 
                                  data_bytes,&search_state[0]);
         break;
      // read/write DATA_MODE register
      case DATA_MODE:
         return_code = DataCommand(cmd,data_length,SIZE_DATA_MODE, 
                                  data_bytes,&mode[0]);
         // check if writing, update MicroLan
         if (data_length != 0)
            UpdateMLanMode(*data_bytes);
         break;
      // read-only registers
      // read DATA_CAPABILITY register
      case DATA_CAPABILITY:
         // make sure reading this read-only registers
         if (data_length == 0)
            return_code = DataCommand(cmd,data_length,SIZE_DATA_CAPABILITY, 
                                  data_bytes,&capability[0]);
         else
            return_code = SingleResult(CMD_ERROR, RET_READ_ONLY);
         break;
      // read DATA_OUTBOUND_MAX register
      case DATA_OUTBOUND_MAX:
         // make sure reading this read-only registers
         if (data_length == 0)
            return_code = DataCommand(cmd,data_length,SIZE_DATA_OUTBOUND_MAX, 
                                  data_bytes,&outbound_max[0]);
         else
            return_code = SingleResult(CMD_ERROR, RET_READ_ONLY);
         break;
      // read DATA_INBOUND_MAX register
      case DATA_INBOUND_MAX:
         // make sure reading this read-only registers
         if (data_length == 0)
            return_code = DataCommand(cmd,data_length,SIZE_DATA_INBOUND_MAX, 
                                  data_bytes,&inbound_max[0]);
         else
            return_code = SingleResult(CMD_ERROR, RET_READ_ONLY);
         break;
      // read DATA_PROTOCOL register
      case DATA_PROTOCOL:
         // make sure reading this read-only registers
         if (data_length == 0)
            return_code = DataCommand(cmd,data_length,(uchar)(strlen(protocol)+1), 
                                  data_bytes,&protocol[0]);
         else
            return_code = SingleResult(CMD_ERROR, RET_READ_ONLY);
         break;
      // read DATA_VENDOR register
      case DATA_VENDOR:
         // make sure reading this read-only registers
         if (data_length == 0)
            return_code = DataCommand(cmd,data_length,(uchar)(strlen(vendor)+1), 
                                  data_bytes,&vendor[0]);
         else
            return_code = SingleResult(CMD_ERROR, RET_READ_ONLY);
         break;
      // multi-byte commands 
      // MicroLan bit operation
      case CMD_ML_BIT: 
         return_code = MLanBitBlock(data_length, data_bytes);
         break;
      // MicroLan byte operation
      case CMD_ML_DATA:
         return_code = MLanByteBlock(data_length, data_bytes);
         break;
      // command not known
      default:
         // check for single-byte command
         if (SINGLE_BYTE_CMD & cmd)
            return_code = SingleResult(cmd, RET_CMD_UNKNOWN);
         else
            return_code = SingleResult(CMD_ERROR, RET_CMD_UNKNOWN);
         break;
   }
   
   return return_code;
}


//----------------------------------------------------------------------
// Read or write a data register identified with the the command 
// code 'cmd'.  
//
//  'cmd'         - the data command code specifying the register
//  'data_length' - number of data bytes to write to register or 0 if 
//                  reading the data register
//  'max_length'  - size of the data register on the repeater, this is
//                  maximum byte length that can be written
//  '*data_bytes' - pointer to the begining of data bytes 
//                  (points into inbound buffer)
//  '*local'      - pointer to the begining of local data register 
//
// Returns:  RET_SUCCESS: data register successfully written or
//                        data register read and placed in outbound 
//                        frame.
//           RET_OUTBOUND_OVERRUN: error, there was not enough 
//                        room in outbound frame.
//           RET_REG_OVERRUN: error, attempt to write more bytes
//                        then the data register could hold
//
uchar DataCommand(uchar cmd, uchar data_length, uchar max_length,
                  uchar *data_bytes, uchar *local)
{
   int i;

   // check if reading 
   if (data_length == 0)
   {
      // check size required in outbound
      if (CheckOutboundSpace((uchar)(max_length + 2)))
      {
         // add command to outbound
         outbound[++outbound[0]] = cmd;

         // add the lenght
         outbound[++outbound[0]] = max_length;

         // add the data itself
         for (i = 0; i < max_length; i++)
             outbound[++outbound[0]] = local[i];
            
         return RET_SUCCESS;
      }
      else
         return RET_OUTBOUND_OVERRUN;
   }
   // writing
   else
   {
      // check if too long for local storage
      if (data_length > max_length)
         return SingleResult(CMD_ERROR, RET_REG_OVERRUN);

      // copy data in inbound to local repeater storage
      for (i = 0; i < data_length; i++)
         local[i] = data_bytes[i];
               
      // zero remainder of register
      for (; i < max_length; i++)
         local[i] = 0;

      return RET_SUCCESS;
   }
}


//----------------------------------------------------------------------
// Check if there is space available for 'num' bytes in the outbound
// buffer.  If space is not available then mark the last valid
// return code to overrun.  Return TRUE if space is available else
// return FALSE.
//
//  'num' - the number of bytes that are requested in the outbound 
//          frame.
//
// Returns: FALSE: if not enough room in the outbound frame for 'num' 
//                 bytes. Error result RET_OUTBOUND_OVERRUN
//                 has been placed in outbound frame.
//          TRUE:  there is room in the outbound frame for 'num'
//                 bytes.  Nothing has been placed in the outbound
//                 frame.
//
int CheckOutboundSpace(uchar num)
{
   // check if enough room in outbound for 'num' bytes
   if ((outbound[0] + num + 2) > MAX_OUTFRAME)   
   {
      // add error command to outbound
      outbound[++outbound[0]] = CMD_ERROR;

      // add the error type
      outbound[++outbound[0]] = RET_OUTBOUND_OVERRUN;

      // set last command
      last_cmd = CMD_ERROR;
      last_cmd_return = RET_OUTBOUND_OVERRUN;
 
      return FALSE;
   }

   return TRUE;
}

//----------------------------------------------------------------------
// If space is available, place the command and return code into
// the outbound buffer.  Returns the return_code passed to this
// functions if space is available else RET_OUTBOUND_OVERRUN is
// returned.
//
//  'cmd'         - the data command code to place in the outbound
//                  frame.
//  'return_code' - the return code to place in the outbound frame.
//
// Returns:  return_code: return the return code passed if room
//                        in outbound frame
//           RET_OUTBOUND_OVERRUN: if not enough room in the outbound
//                        frame for these two bytes.
//
uchar SingleResult(uchar cmd, uchar return_code)
{
   // add the command
   outbound[++outbound[0]] = cmd;

   // add the return code
   outbound[++outbound[0]] = return_code;

   // set last command
   last_cmd = cmd;
   last_cmd_return = return_code;

   // pass along the return code for halt check
   return return_code;
}


//----------------------------------------------------------------------
// Perform the repeater master reset.  Set all data objects
// to the default state.
//
// Returns:  RET_SUCCESS: repeater has been reset to default state.
//
uchar MLanRepeaterReset(void)
{
   int i;

   // set data registers to default
   for (i = 0; i < SIZE_DATA_ID; i++)
      id[i] = 0;
   for (i = 0; i < SIZE_DATA_SEARCH_STATE; i++)
      search_state[i] = 0;

   search_cmd[0] = SEARCH_NORMAL;
   mode[0] = MODE_NORMAL;
   capability[0] = MODE_OVERDRIVE | MODE_STRONG5 | MODE_PROGRAM;
   outbound_max[0] = MAX_OUTFRAME;
   inbound_max[0] = MAX_INFRAME;
   sprintf(&protocol[0],"ML100");
   sprintf(&vendor[0],"DALSEMI");
   outbound[0] = 0;
   last_cmd = CMD_ML_RESET;
   last_cmd_return = RET_SUCCESS;
   LastDeviceFlag = 0;

   return RET_SUCCESS;
}


//----------------------------------------------------------------------
// Scan through 'num' bytes from the provided inbound buffer
// to look for a CMD_GETBUF command.  This is used to determine
// if the buffer should be returned on an error condition.
// Return TRUE if CMD_GETBUF is found else return FALSE.
//
//  'buffer' - pointer to the location in the inbound buffer
//             that has not been processes yet.
//  'num'    - the number of bytes that remain in the inbound 
//             buffer to check.
//
// Returns: FALSE: CMD_GETBUF command not found
//          TRUE:  CMD_GETBUF command found
//
int ScanGETBUF(uchar *buffer, uchar num)
{
   uchar in_cnt = 0, cmd, data_length;

   // loop to process through the inbound packet
   while (in_cnt < (num + 1))
   {
      // retrieve an inbound buffer byte, command 
      cmd = buffer[in_cnt++];

      // check for multi-byte command
      if (!(SINGLE_BYTE_CMD & cmd))
      {
         // retrieve an buffer byte, data length 
         data_length = buffer[in_cnt++];

         // adjust the buffer count
         in_cnt += data_length;
      }
      // check for the CMD_GETBUF function
      else if (cmd == CMD_GETBUF)
         return TRUE;
   }

   return FALSE;
} 

