//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  TMEXLnkP.C - Link module to call on TMEX low-level functions to
//               excersize the MicroLan (MLan) repeater packet 
//               functions.
//
//  Version: 1.00
//

#include <windows.h>
#include "mlanpkt.h"

// external TMEX variables and functions
extern int SessionHandle;
extern short far pascal TMTouchByte(long, short);
extern short far pascal TMTouchReset(long);
extern short far pascal TMTouchBit(long, short);
extern short far pascal TMOneWireCom(long, short, short);
extern short far pascal TMOneWireLevel(long, short, short, short);

// exportable low-level functions
void UpdateMLanMode(uchar);
uchar MLanDelay(uchar);
uchar MLanTouchReset(void);
uchar MLanTouchBit(uchar);
uchar MLanTouchByte(uchar);

//--------------------------------------------------------------------------
// Reset all of the devices on the MicroLan and return the result.
//
// Returns: RET_SUCCESS:   presense pulse(s) detected, device(s) reset
//          RET_NO_DEVICE: no presense pulses detected
//
uchar MLanTouchReset(void)
{
   short result;

   // Assume valid Session
   result = TMTouchReset(SessionHandle);

   // success if the normal or alarm presence
   if ((result == 1) || (result == 2))
      return RET_SUCCESS;
   else
      return RET_NO_DEVICE;
}


//--------------------------------------------------------------------------
// Send 1 bit of communication to the MicroLan and return the
// result 1 bit read from the MicroLan.  The parameter 'sendbit'
// least significant bit is used and the least significant bit
// of the result is the return bit.
//
// 'sendbit' - the least significant bit is the bit to send
//
// Returns: 0:   0 bit read from sendbit
//          1:   1 bit read from sendbit
//
uchar MLanTouchBit(uchar sendbit)
{
   // Assume valid Session
   return (uchar)TMTouchBit(SessionHandle,(short)sendbit);
}


//--------------------------------------------------------------------------
// Send 8 bits of communication to the MicroLan and return the
// result 8 bits read from the MicroLan.  The parameter 'sendbyte'
// least significant 8 bits are used and the least significant 8 bits
// of the result is the return byte.
//
// 'sendbyte' - 8 bits to send
//
// Returns:  8 bytes read from sendbyte
//
uchar MLanTouchByte(uchar sendbyte)
{
   // Assume valid Session
   return (uchar)TMTouchByte(SessionHandle,(short)sendbyte);
}


//--------------------------------------------------------------------------
// Set the speed and level of the MicroLan based on the NewMode
// value.  The values for NewMode are described with the following
// bit masks.
//
// MODE_NORMAL     0x00
// MODE_OVERDRIVE  0x01
// MODE_STRONG5    0x02
// MODE_PROGRAM    0x04
// MODE_BREAK      0x08
//
void UpdateMLanMode(uchar NewMode)
{
   // set speed 
   if (NewMode & MODE_OVERDRIVE)
      TMOneWireCom(SessionHandle, 0, 0x01);
   else
      TMOneWireCom(SessionHandle, 0, 0x00);

   // set level
   if (NewMode & MODE_STRONG5)
      TMOneWireLevel(SessionHandle, 0, 0x01, 0);
   else if (NewMode & MODE_PROGRAM)
      TMOneWireLevel(SessionHandle, 0, 0x03, 0);
   else if (NewMode & MODE_BREAK)
      TMOneWireLevel(SessionHandle, 0, 0x02, 0);
   else
      TMOneWireLevel(SessionHandle, 0, 0x00, 0);
}


//--------------------------------------------------------------------------
// Delay for a specified 'DelayFlag' length of time.  The time values
// are defined in "mlanpkt.h". The most significant bit indicates if
// the values is in milliseconds (set) or microseconds (not set) 
//
// DELAY_32        0x00
// DELAY_64        0x01
// DELAY_128       0x02
// DELAY_256       0x03
// DELAY_512       0x04
// DELAY_1024      0x05
// DELAY_2048      0x06
// DELAY_4096      0x07
//
// Returns:  RET_SUCCESS  - delay operation complete
//
uchar MLanDelay(uchar DelayFlag)
{
   int delay;

   // check if units in Milliseconds
   if (DelayFlag & DELAY_MS)
   {
      // convert to ms
      delay = 1 << (5 + (DelayFlag - DELAY_MS)); 
      
      Sleep(delay);
   }
   else
   {
      // convert to ms
      delay = (1 << (5 + DelayFlag)) / 1000;
      // for test implementation, if < 1ms then default to 1ms
      if (!delay)
         delay = 1;

      Sleep(delay);
   }

   return RET_SUCCESS;
}
