//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  TranNet.C - MicroLAN Transport and Network functions for repeater 
//              packet processing.
//
//  Version: 1.00
//

#include "mlanpkt.h"

// global used in CRC calculation
static uchar DOWCRC;

// external Link MicroLAN functions
extern uchar MLanDelay(uchar);
extern uchar MLanTouchReset(void);
extern uchar MLanTouchBit(uchar);
extern uchar MLanTouchByte(uchar);
extern void UpdateMLanMode(uchar);

// packet processing functions
extern uchar SingleResult(uchar, uchar); 
extern int   CheckOutboundSpace(uchar);

// Transport and Network MicroLAN functions
uchar MLanSearch(void);
uchar MLanAccess(void);
uchar MLanOverdriveAccess(void);
uchar MLanBitBlock(uchar, uchar *);
uchar MLanByteBlock(uchar, uchar *);
uchar dowcrc(uchar);

//----------------------------------------------------------------------
//  Write and read the block of bit commands.
//
//  'data_length' - number of data bytes provided that represent
//                  the number of bit operations to do
//  '*data_bytes' - pointer to the begining of data bytes 
//                  (points into inbound buffer)
//
// Returns:  RET_SUCCESS: bit operations successfully done and
//                        the results have been placed in the 
//                        outbound frame.
//           RET_OUTBOUND_OVERRUN: error, there was not enough 
//                        room in outbound frame.
//
uchar MLanBitBlock(uchar data_length, uchar *data_bytes)
{
   int i;

   // 0 length not valid
   if (data_length == 0)
      return SingleResult(CMD_ERROR,RET_WRITE_ONLY); 

   // check size required in outbound
   if (CheckOutboundSpace((uchar)(data_length + 2)))
   {
      // add command to outbound
      outbound[++outbound[0]] = CMD_ML_BIT;

      // add the lenght
      outbound[++outbound[0]] = data_length;

      // add the data itself
      for (i = 0; i < data_length; i++)
         outbound[++outbound[0]] = MLanTouchBit((uchar)(data_bytes[i] & 0x01));

      return RET_SUCCESS;
   }
   else
      return RET_OUTBOUND_OVERRUN;
}


//----------------------------------------------------------------------
//  Write and read a block of bytes to the MicroLan.  The first
//  data byte is a length byte that represents the total number
//  of data byte transactions that are to be done.  If this
//  exceeds the number of data bytes available then the remaining
//  byte communications are slave reads represented by the
//  master writing 0xFF and reading the result. 
//
//  'data_length' - number of data bytes provided.  The first
//                  data byte is a length byte.
//  '*data_bytes' - pointer to the begining of data bytes 
//                  (points into inbound buffer)
//
// Returns:  RET_SUCCESS: MicroLan communication successfully done and
//                        the results have been placed in the 
//                        outbound frame.
//           RET_OUTBOUND_OVERRUN: error, there was not enough 
//                        room in outbound frame.
//
//
uchar MLanByteBlock(uchar data_length, uchar *data_bytes)
{
   int i;
   uchar wr_len;
   
   // 0 length not valid
   if (data_length == 0)
      return SingleResult(CMD_ERROR,RET_WRITE_ONLY); 

   // get the write length byte
   wr_len = data_bytes[0];

   // check size required in outbound
   if (CheckOutboundSpace((uchar)(wr_len + 2)))
   {
      // add command to outbound
      outbound[++outbound[0]] = CMD_ML_DATA;

      // add the lenght
      outbound[++outbound[0]] = wr_len;

      // add the data itself
      for (i = 0; i < wr_len; i++)
      {
         if (i < (data_length - 1))
            outbound[++outbound[0]] = MLanTouchByte(data_bytes[i + 1]);
         else
            outbound[++outbound[0]] = MLanTouchByte(0xFF);
      }

      return RET_SUCCESS;
   }
   else
      return RET_OUTBOUND_OVERRUN;
}


//----------------------------------------------------------------------
// Perform a MATCH command to select the 1-Wire device with the
// address in the ID data register.
//
// Returns:  RET_SUCCESS: If the device is present on the MicroLAN
//                        then the device is selected.
//           RET_NO_DEVICE: Error, no devices present on the MicroLAN
//           RET_ML_SHORTED: Error, MicroLAN shorted, communication not
//                        possible.
//
uchar MLanAccess(void)
{
   int i;
   uchar return_code;

   // call the MicroLan reset function 
   return_code = MLanTouchReset();
   if (return_code == RET_SUCCESS)
   {
      // send the match command 0x55
      if (MLanTouchByte(0x55) == 0x55)
      {
         // loop to send the DATA_ID
         for (i = 0; i < 8; i++)
         {
            if (MLanTouchByte(id[i]) != id[i])
               return RET_ML_SHORTED;
         }
      }
      else
         return RET_ML_SHORTED;
   }
   else
      return RET_NO_DEVICE;
   
   return RET_SUCCESS;
}


//----------------------------------------------------------------------
// Perform a MATCH command to select the 1-Wire device with the
// address in the ID data register.
//
// Returns:  RET_SUCCESS: If the device is present on the MicroLAN
//                        then the device is selected.
//           RET_ERROR:   Error, this repeater does not support overdrive
//           RET_NO_DEVICE: Error, no devices present on the MicroLAN
//           RET_ML_SHORTED: Error, MicroLAN shorted, communication not
//                        possible.
//
uchar MLanOverdriveAccess(void)
{
   int i;
   uchar return_code;

   // check for overdrive capability
   if (!(capability[0] & MODE_OVERDRIVE))
      return RET_ERROR;

   // force repeater to normal communication speed
   mode[0] = MODE_NORMAL;
   UpdateMLanMode(mode[0]);

   // call the MicroLan reset function 
   return_code = MLanTouchReset();
   if (return_code == RET_SUCCESS)
   {
      // send the match command 0x69
      if (MLanTouchByte(0x69) == 0x69)
      {
         // switch repeater to overdrive communication speed
         mode[0] = MODE_OVERDRIVE;
         UpdateMLanMode(mode[0]);

         // loop to send the DATA_ID in overdrive
         for (i = 0; i < 8; i++)
         {
            if (MLanTouchByte(id[i]) != id[i])
               return RET_ML_SHORTED;
         }
      }
      else
         return RET_ML_SHORTED;
   }
   else
      return RET_NO_DEVICE;
   
   return RET_SUCCESS;
}


//--------------------------------------------------------------------------
// The 'MLanSearch' function does a general search.  This function
// continues from the previous search state. The search state
// can be reset by setting the search state variables to 0.  For
// a normal search, the CMD_ML_RESET command must be called before
// this search function.
//
// Returns:  RET_SUCCESS: when a 1-Wire device was found and it's 
//                        ROM number placed in the global ROM
//           RET_END_SEARCH: when no new device was found.  Either 
//                        the last search was the last device or 
//                        there are no devices on the MicroLan.
// 
uchar MLanSearch(void)
{
   int id_bit_number;
   int last_zero, ROM_byte_number;
   uchar ROM_byte_mask, search_direction;
   uchar *LastDiscrepancy, *LastFamilyDiscrepancy, *ROM;
   uchar id_bit, cmp_id_bit;

   // initialize for search 
   id_bit_number = 1;
   last_zero = 0;
   ROM_byte_number = 0;
   ROM_byte_mask = 1;
   DOWCRC = 0;         

   // set pointers to data space variables
   LastDiscrepancy = &search_state[0];
   LastFamilyDiscrepancy = &search_state[1];
   ROM = &id[0];
   
   // if the last call was not the last one 
   if (!LastDeviceFlag)
   {
      // send the search command
      MLanTouchByte(search_cmd[0]); 

      // loop to do the search rom 
      do
      {
         // read an ID bit and its compliment 
         id_bit = MLanTouchBit(1);
         cmp_id_bit = MLanTouchBit(1);

         // check for no devices on 1-wire
         if ((id_bit == 1) && (cmp_id_bit == 1))  
            break;
         // check both 0 and 1 in this bit position (discrepancy)
         else if ((id_bit == 0) && (cmp_id_bit == 0))
         {
            // are we at the last discrepancy (go down 1 path)
            if (id_bit_number == *LastDiscrepancy)
               search_direction = 1;
            // are we after the last discrepancy (go down 0 path)
            else if (id_bit_number > *LastDiscrepancy)
               search_direction = 0;
            // we are before the last descrepancy (pick same path)
            else
               search_direction = ((ROM[ROM_byte_number] & ROM_byte_mask) > 0);

            // if 0 was picked then record its position in LastZero 
            if (search_direction == 0) 
            {
               // record this 0 path location 
               last_zero = id_bit_number;  

               // check for Last discrepancy in family 
               if (last_zero < 9)
                  *LastFamilyDiscrepancy = last_zero;
            }

         }
         // all devices coupled have 0 or 1
         else            
            search_direction = id_bit;  // bit write value for search 

         // set or clear the bit in the ROM byte ROM_byte_number 
         // with mask ROM_byte_mask 
         if (search_direction == 1)
           ROM[ROM_byte_number] |= ROM_byte_mask;
         else
           ROM[ROM_byte_number] &= ~ROM_byte_mask;

         // rom search direction write bit 
         MLanTouchBit(search_direction);

         // increment the byte counter id_bit_number 
         // and shift the mask ROM_byte_mask 
         id_bit_number++;
         ROM_byte_mask <<= 1;

         // if the mask is 0 then go to new ROM byte ROM_byte_number
         // and reset mask 
         if (ROM_byte_mask == 0)
         {
             dowcrc(ROM[ROM_byte_number]);  // accumulate the CRC 
             ROM_byte_number++; 
             ROM_byte_mask = 1;
         }
      } 
      while (ROM_byte_number < 8);  // loop until through all ROM bytes 0-7 

      // store the last zero path
      *LastDiscrepancy = last_zero;
      // if no 0 discrepency then last device
      if (*LastDiscrepancy == 0)
         LastDeviceFlag = TRUE;
   }
   
   // if no device found then reset counters so next 'next' will be
   // like a first 
   if (DOWCRC || !ROM[0] || (id_bit_number < 65))
   {
      *LastDiscrepancy = 0;
      LastDeviceFlag = FALSE;
      *LastFamilyDiscrepancy = 0; 
      return RET_END_SEARCH;
   }
   else
      return RET_SUCCESS;
}


//--------------------------------------------------------------------------
// Update the Dallas Semiconductor One Wire CRC (DOWCRC) from the global
// variable DOWCRC and the argument.  Return the updated DOWCRC.
//
uchar dscrc_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};

uchar dowcrc(uchar x)
{
   DOWCRC = dscrc_table[DOWCRC ^ x];
   return DOWCRC;
}