//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  MLanLLU.C - Link Layer MicroLan functions using the DS2480 (U)
//              serial interface chip.
//
//  Version: 1.00
//
//

#include "DS2480.h"
#include "mlanpkt.h"
#include <windows.h> 

// external COM functions required
extern void FlushCOM(void);
extern int  WriteCOM(int, uchar *);
extern int  ReadCOM(int, uchar *);
extern void SetBaudCOM(uchar);

// external DS2480 utility function
extern int DS2480Detect(void);
extern int DS2480ChangeBaud(uchar);

// local functions
uchar MLanTouchReset(void);
uchar MLanTouchBit(uchar sendbit);
uchar MLanTouchByte(uchar sendbyte);
uchar MLanDelay(uchar);
void UpdateMLanMode(uchar);
uchar MLanSpeed(uchar);
uchar MLanLevel(uchar);

// external globals
extern uchar UMode;   // current DS2480 command or data mode state
extern uchar UBaud;   // current DS2480 baud rate
extern uchar USpeed;  // current DS2480 MicroLAN communication speed
extern uchar ULevel;  // current DS2480 MicroLAN level

//--------------------------------------------------------------------------
// Reset all of the devices on the MicroLan and return the result.
//
// Returns: RET_SUCCESS:   presense pulse(s) detected, device(s) reset
//          RET_NO_DEVICE: no presense pulses detected
//
// WARNING: This routine will not function correctly on some
//          Alarm reset types of the DS1994/DS1427/DS2404 with
//          Rev 1 and 2 of the DS2480.
//
uchar MLanTouchReset(void)
{
   uchar readbuffer[10],sendpacket[10];
   int sendlen=0;

   // make sure normal level
   MLanLevel(MODE_NORMAL);

   // check if correct mode 
   if (UMode != MODSEL_COMMAND)
   {
      UMode = MODSEL_COMMAND;
      sendpacket[sendlen++] = MODE_COMMAND;
   }

   // construct the command
   sendpacket[sendlen++] = (uchar)(CMD_COMM | FUNCTSEL_RESET | USpeed);

   // flush the buffers
   FlushCOM();

   // send the packet 
   if (WriteCOM(sendlen,sendpacket)) 
   {
      // read back the 1 byte response 
      if (ReadCOM(1,readbuffer) == 1)
      {
         // make sure this byte looks like a reset byte
         if (((readbuffer[0] & 0xC0) == 0xC0) &&
              (((readbuffer[0] & RB_RESET_MASK) == RB_PRESENCE) ||
               ((readbuffer[0] & RB_RESET_MASK) == RB_ALARMPRESENCE))) 
            return RET_SUCCESS;
         else
            return RET_NO_DEVICE;
      }
   }

   // an error occured so re-sync with DS2480
   DS2480Detect();

   return RET_NO_DEVICE;
}


//--------------------------------------------------------------------------
// Send 1 bit of communication to the MicroLan and return the
// result 1 bit read from the MicroLan.  The parameter 'sendbit'
// least significant bit is used and the least significant bit
// of the result is the return bit.
//
// 'sendbit' - the least significant bit is the bit to send
//
// Returns: 0:   0 bit read from sendbit
//          1:   1 bit read from sendbit
//
uchar MLanTouchBit(uchar sendbit)
{
   uchar readbuffer[10],sendpacket[10];
   int sendlen=0;

   // make sure normal level
   MLanLevel(MODE_NORMAL);

   // check if correct mode 
   if (UMode != MODSEL_COMMAND)
   {
      UMode = MODSEL_COMMAND;
      sendpacket[sendlen++] = MODE_COMMAND;
   }

   // construct the command
   sendpacket[sendlen] = (sendbit != 0) ? BITPOL_ONE : BITPOL_ZERO;
   sendpacket[sendlen++] |= CMD_COMM | FUNCTSEL_BIT | USpeed;

   // flush the buffers
   FlushCOM();

   // send the packet 
   if (WriteCOM(sendlen,sendpacket)) 
   {
      // read back the response 
      if (ReadCOM(1,readbuffer) == 1)
      {
         // interpret the response 
         if (((readbuffer[0] & 0xE0) == 0x80) &&
             ((readbuffer[0] & RB_BIT_MASK) == RB_BIT_ONE))
            return 1;
         else
            return 0;
      }
   }

   // an error occured so re-sync with DS2480
   DS2480Detect();

   return 0;
}


//--------------------------------------------------------------------------
// Send 8 bits of communication to the MicroLan and return the
// result 8 bits read from the MicroLan.  The parameter 'sendbyte'
// least significant 8 bits are used and the least significant 8 bits
// of the result is the return byte.
//
// 'sendbyte' - 8 bits to send
//
// Returns:  8 bytes read from sendbyte
//
uchar MLanTouchByte(uchar sendbyte)
{
   uchar readbuffer[10],sendpacket[10];
   int sendlen=0;

   // make sure normal level
   MLanLevel(MODE_NORMAL);

   // check if correct mode 
   if (UMode != MODSEL_DATA)
   {
      UMode = MODSEL_DATA;
      sendpacket[sendlen++] = MODE_DATA;
   }

   // add the byte to send
   sendpacket[sendlen++] = (uchar)sendbyte;

   // check for duplication of data that looks like COMMAND mode 
   if (sendbyte == MODE_COMMAND) 
      sendpacket[sendlen++] = (uchar)sendbyte;

   // flush the buffers
   FlushCOM();

   // send the packet 
   if (WriteCOM(sendlen,sendpacket)) 
   {
      // read back the 1 byte response 
      if (ReadCOM(1,readbuffer) == 1)
      {
          // return the response 
          return readbuffer[0];
      }
   }

   // an error occured so re-sync with DS2480
   DS2480Detect();

   return 0;
}


//--------------------------------------------------------------------------
// Delay for a specified 'DelayFlag' length of time.  The time values
// are defined in "mlanpkt.h". The most significant bit indicates if
// the values is in milliseconds (set) or microseconds (not set) 
//
// DELAY_32        0x00
// DELAY_64        0x01
// DELAY_128       0x02
// DELAY_256       0x03
// DELAY_512       0x04
// DELAY_1024      0x05
// DELAY_2048      0x06
// DELAY_4096      0x07
//
// Returns:  RET_SUCCESS  - delay operation complete
//
uchar MLanDelay(uchar DelayFlag)
{
   int delay;

   // check if units in Milliseconds
   if (DelayFlag & DELAY_MS)
   {
      // convert to ms
      delay = 1 << (5 + (DelayFlag - DELAY_MS)); 
      
      Sleep(delay);
   }
   else
   {
      // convert to ms
      delay = (1 << (5 + DelayFlag)) / 1000;
      // for test implementation, if < 1ms then default to 1ms
      if (!delay)
         delay = 1;

      Sleep(delay);
   }

   return RET_SUCCESS;
}


//--------------------------------------------------------------------------
// Set the speed and level of the MicroLan based on the NewMode
// value.  The values for NewMode are described with the following
// bit masks.
//
// MODE_NORMAL     0x00
// MODE_OVERDRIVE  0x01
// MODE_STRONG5    0x02
// MODE_PROGRAM    0x04
// MODE_BREAK      0x08
//
void UpdateMLanMode(uchar NewMode)
{
   // set speed
   if (NewMode & MODE_OVERDRIVE)
      MLanSpeed(MODE_OVERDRIVE);
   else
      MLanSpeed(MODE_NORMAL);

   // set level
   if (NewMode & MODE_STRONG5)
      MLanLevel(MODE_STRONG5);
   else if (NewMode & MODE_PROGRAM)
      MLanLevel(MODE_PROGRAM);
   else if (NewMode & MODE_BREAK)
      MLanLevel(MODE_BREAK);
   else
      MLanLevel(MODE_NORMAL);
}


//--------------------------------------------------------------------------
// Set the MicroLAN communucation speed.  The values for NewSpeed are
// as follows:
//
// MODE_NORMAL     0x00
// MODE_OVERDRIVE  0x01
//
// Returns:  current MicroLAN speed 
//
uchar MLanSpeed(uchar NewSpeed)
{
   uchar sendpacket[5];
   short sendlen=0;
   int rt = FALSE;
   
   // check if change from current mode
   if (((NewSpeed == MODE_OVERDRIVE) &&
        (USpeed != SPEEDSEL_OD)) ||
       ((NewSpeed == MODE_NORMAL) &&
        (USpeed != SPEEDSEL_FLEX)))
   {
      if (NewSpeed == MODE_OVERDRIVE) 
      {
         // if overdrive then switch to 57600 baud
         if (DS2480ChangeBaud(PARMSET_57600) == PARMSET_57600)
         {
            USpeed = SPEEDSEL_OD;
            rt = TRUE;
         }
      }
      else if (NewSpeed == MODE_NORMAL) 
      {
         // else normal so set to 9600 baud
         if (DS2480ChangeBaud(PARMSET_9600) == PARMSET_9600)
         {
            USpeed = SPEEDSEL_FLEX;
            rt = TRUE;
         }
      }

      // if baud rate is set correctly then change DS2480 speed
      if (rt)
      {
         // check if correct mode 
         if (UMode != MODSEL_COMMAND)
         {
            UMode = MODSEL_COMMAND;
            sendpacket[sendlen++] = MODE_COMMAND;
         }

         // proceed to set the DS2480 communication speed
         sendpacket[sendlen++] = CMD_COMM | FUNCTSEL_SEARCHOFF | USpeed;

         // send the packet 
         if (!WriteCOM(sendlen,sendpacket)) 
         {
            rt = FALSE;
            // lost communication with DS2480 then reset 
            DS2480Detect();
         }
      }
   }

   // return the current speed
   return (USpeed == SPEEDSEL_OD) ? MODE_OVERDRIVE : MODE_NORMAL;
}


//--------------------------------------------------------------------------
// Set the MicroLAN line level.  The values for NewLevel are
// as follows:
//
// MODE_NORMAL     0x00
// MODE_STRONG5    0x02
// MODE_PROGRAM    0x04
// MODE_BREAK      0x08 (not supported)
//
// Returns:  current MicroLAN level  
//
uchar MLanLevel(uchar NewLevel)
{
   uchar sendpacket[10],readbuffer[10];
   short sendlen=0;
   short rt=FALSE;

   // check if need to change level
   if (NewLevel != ULevel)
   {
      // check if just putting back to normal
      if (NewLevel == MODE_NORMAL)
      {
         // check if correct mode 
         if (UMode != MODSEL_COMMAND)
         {
            UMode = MODSEL_COMMAND;
            sendpacket[sendlen++] = MODE_COMMAND;
         }

         // stop pulse command
         sendpacket[sendlen++] = MODE_STOP_PULSE;
   
         // flush the buffers
         FlushCOM();

         // send the packet 
         if (WriteCOM(sendlen,sendpacket)) 
         {
            // read back the 1 byte response 
            if (ReadCOM(1,readbuffer) == 1)
            {
               // check response byte
               if ((readbuffer[0] & 0xE0) == 0xE0)
               {
                  rt = TRUE;
                  ULevel = MODE_NORMAL;
               }
            }
         }
      }
      // set new level
      else
      {
         // check if correct mode 
         if (UMode != MODSEL_COMMAND)
         {
            UMode = MODSEL_COMMAND;
            sendpacket[sendlen++] = MODE_COMMAND;
         }

         // strong 5 volts
         if (NewLevel == MODE_STRONG5)
         {
            // set the SPUD time value 
            sendpacket[sendlen++] = CMD_CONFIG | PARMSEL_5VPULSE | PARMSET_infinite;
            // add the command to begin the pulse
            sendpacket[sendlen++] = CMD_COMM | FUNCTSEL_CHMOD | SPEEDSEL_PULSE | BITPOL_5V;
         }
         // 12 volts
         else if (NewLevel == MODE_PROGRAM)
         {
            // set the PPD time value 
            sendpacket[sendlen++] = CMD_CONFIG | PARMSEL_12VPULSE | PARMSET_infinite;
            // add the command to begin the pulse
            sendpacket[sendlen++] = CMD_COMM | FUNCTSEL_CHMOD | SPEEDSEL_PULSE | BITPOL_12V;
         }

         // flush the buffers
         FlushCOM();

         // send the packet 
         if (WriteCOM(sendlen,sendpacket)) 
         {
            // read back the 1 byte response from setting time limit
            if (ReadCOM(1,readbuffer) == 1)
            {
               // check response byte
               if ((readbuffer[0] & 0x81) == 0)
               {
                  ULevel = NewLevel;
                  rt = TRUE;
               }
            }
         }
      }

      // if lost communication with DS2480 then reset 
      if (rt != TRUE)
         DS2480Detect();
   }

   // return the current level
   return ULevel;      
}

